#define DEBUG

#include <limits.h>
#include <math.h>

#include <SPI.h>

#include <avr/sleep.h>

#include <ST7735.h>
#include <fonts/allFonts.h>


// Pin definitions
#define SCLK 13
#define MISO 12
#define MOSI 11
#define CS_DISP 10
#define RST_DISP 9
#define DC_DISP 8
#define CS_SD 4
#define DISPON A2

// for compatibility to the TFT18 lib
#define sclk SCLK
#define mosi MOSI
#define cs CS_DISP
#define dc DC_DISP
#define rst RST_DISP

volatile uint8_t displayOnState=0;


// Arduino 16MHz multiply speed times
// byte*byte: 0.4µs
// int*int:   0.9µs
// long*long: 3.8µs
// float*float: 9.2µs
// float/float: 34µs
// sin():       255µs
// float: 4 bytes
// int: 2 bytes
// double: 4 bytes
// int/int: 15µs


// TFT 1.8'display: 128x160 resolution
ST7735 tft = ST7735(CS_DISP, DC_DISP, RST_DISP);    


void display_on(uint16_t startColor)  {
  if (!displayOnState)  {
    pinMode(DISPON, OUTPUT);  // Display ON    
    digitalWrite(DISPON, LOW);
    delay(100);
    
    // we use hardware SPI interface
    SPI.begin();
    SPI.setClockDivider(SPI_CLOCK_DIV2);
    SPI.setDataMode(SPI_MODE3);  
    
    tft.initR();               // initialize a ST7735R chip (display controller)
    tft.writecommand(ST7735_DISPON);
    tft.fillScreen(startColor);
    tft.setBackgroundColor(startColor);        
    displayOnState=1;
  }
}

void display_off()  {
  if (displayOnState)  {
    SPI.end();
    // set all signals to low to avoid current via ESD diodes
    digitalWrite(CS_DISP, LOW);
    digitalWrite(RST_DISP, LOW);
    digitalWrite(DC_DISP, LOW);
    digitalWrite(SCLK, LOW);
    digitalWrite(MOSI, LOW);
    digitalWrite(DISPON, HIGH); // Display OFF 
    displayOnState=0;
  }
}

char *ftoa(char *a, float f, unsigned int precision)
{
  if (precision>4) precision=4;  // prevent overflow
  
  char *ret = a;
  int heiltal = (int)f;
  itoa(heiltal, a, 10);
  while (*a != '\0') a++;
  *a++ = '.';
  // need leading zeros for fractional numbers

  float tmpf=f-heiltal; 
  for (unsigned int i=0; i<precision; i++)  {
    tmpf=10*tmpf;
    *a++ = '0'+ abs((int)(tmpf));
    tmpf=tmpf-int(tmpf);
  }
  *a++ = 0;  // string ends
  return ret;
}

void setup() {        
#ifdef DEBUG  
  Serial.begin(9600);
#endif

  pinMode(DISPON, OUTPUT);  // Display ON
  digitalWrite(DISPON, HIGH);

  pinMode(CS_SD, OUTPUT);  // chip select for SD card
  digitalWrite(CS_SD, LOW);

}

//
// Arduiono Loop....
//

void loop() {

  char line[22];
  char number[12];

  display_on(YELLOW);

  set_sleep_mode(SLEEP_MODE_IDLE);
  sleep_enable();

  while (1)
  {
    uint8_t tmp,x;
    
    tft.selectFont(Arial_14);
    strcpy(line,"Arial_14");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 0, line, BLACK);

    tft.selectFont(Arial_bold_14);
    strcpy(line,"Arial_bold_14");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 15, line, BLACK);

    tft.selectFont(Corsiva_12);
    strcpy(line,"Corsiva_12");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 30, line, MAGENTA);

    tft.selectFont(CalBlk36);
    strcpy(line,"CalBlk");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 43, line, RED);

    tft.selectFont(CalLite24);
    strcpy(line,"CalLite24");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 75, line, GREEN);

    tft.selectFont(Verdana24);
    strcpy(line,"0123456");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 100, line, BLUE);


    tft.selectFont(fixednums7x15);
    strcpy(line,"0123456");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 125, line, BLACK);

    tft.selectFont(System5x7);
    strcpy(line,"System5x7");
    tmp=tft.stringWidth(line);
    tmp=(tft.width-tmp)/2;
    x=tft.putS(tmp, 140, line, BLACK);

/*
unsigned long StartTime = millis();

for (int i=0; i<1000; i++)
    x=tft.putS(tmp, 0, line, BLACK);

unsigned long CurrentTime = millis();
unsigned long ElapsedTime = CurrentTime - StartTime;

#ifdef DEBUG       

      Serial.println();
      Serial.print("time: ");
      Serial.println(ElapsedTime);
#endif      
*/

#ifdef DEBUG       
      Serial.println(x);

#endif
    //sleep_cpu();
    while(1);
  }  // while(1)
}

